package higa.jhcl.control;

import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import higa.util.StringUtil;
import higa.util.ActionMulticaster;

public class HButtonBar extends HPanel {

  private String[] labels = new String[0];
  private boolean flat = true;

  private HButton[] buttons = new HButton[0];
  private GridLayout gridLayout1 = new GridLayout();

  private ActionMulticaster actionCaster =
    new ActionMulticaster();

  public HButtonBar() {
    try {
      jhInit();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public synchronized void setLabels(String[] newLabels) {
    labels = StringUtil.copy(newLabels);
    newRebuild();
  }

  public String[] getLabels() {
    return StringUtil.copy(labels);
  }

  public int getLabelIndex(String label) {
    int count = labels.length;
    for (int i = 0; i < count; i++) {
      if (labels[i] != null &&
        labels[i].equals(label)) {

        return i;
      }
    }

    return -1;
  }

  public void setRows(int rows) {
    gridLayout1.setRows(rows);
  }

  public int getRows() {
    return gridLayout1.getRows();
  }

  public void setButtonEnabled(
    int button, boolean enabled) {

    buttons[button].setEnabled(enabled);
  }

  public boolean isButtonEnabled(int button) {
    return buttons[button].isEnabled();
  }

  public void setButtonVisible(
    int button, boolean visible) {

    buttons[button].setVisible(visible);
    rebuild();
  }

  public boolean isButtonVisible(int button) {
    return buttons[button].isVisible();
  }

  public void setFlat(boolean flat) {
    this.flat = flat;
    int count = buttons.length;
    for (int i = 0; i < count; i++) {
      buttons[i].setFlat(flat);
    }

    if (flat) {
      setBevelOuter(LOWERED);
      setBevelInner(RAISED);
    } else {
      setBevelOuter(FLAT);
      setBevelInner(FLAT);
    }
  }

  public boolean isFlat() {
    return flat;
  }

  public void addActionListener(ActionListener listener) {
    actionCaster.add(listener);
  }

  public void removeActionListener(ActionListener listener) {
    actionCaster.remove(listener);
  }

  protected synchronized void rebuild() {
    removeAll();
    int count = labels.length;
    for (int i = 0; i < count; i++) {
      if (buttons[i].isVisible()) {
        add(buttons[i]);
      }
    }
  }

  protected synchronized void newRebuild() {
    removeAll();
    int count = labels.length;
    buttons = new HButton[count];
    for (int i = 0; i < count; i++) {
      buttons[i] = addTextButton(labels[i]);
    }

    setFlat(flat);
  }

  protected HButton addTextButton(String label) {
    HButton button = new HButton(label);
    add(button);
    button.addActionListener(actionCaster);
    return button;
  }

  private void jhInit() throws Exception {
    setRows(1);
    this.setLayout(gridLayout1);
    setFlat(true);
  }
}
