package higa.jhcl.view;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Color;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemPaintSite;

public class FocusableItemPainter
  implements ItemPainter, Serializable {

  transient private ItemPainter painter;

  public FocusableItemPainter() {
    this.painter = null;
  }

  public FocusableItemPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public void setPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public ItemPainter getPainter() {
    return painter;
  }

  public Dimension getPreferredSize(
    Object object, Graphics g,
    int state, ItemPaintSite site) {

    return painter != null ?
      painter.getPreferredSize(
      object, g, state, site) :
      new Dimension(0,0);
  }

  public void paint(Object object, Graphics g,
    Rectangle r, int state, ItemPaintSite site) {
    if (painter != null) {
      painter.paint(object, g, r, state, site);
    }

    if ((state & FOCUSED) != 0) {
      drawFocusRect(g, r.x, r.y, r.width, r.height);
    }
  }

  protected void drawDottedHLine(Graphics g,
    int y0, int x0, int x1, int dotSpacing) {
    for (int x = ((x0 + 1) / 2) * 2; x <= x1; x += dotSpacing) {
      g.drawLine(x, y0, x, y0);
    }
  }

  protected void drawDottedVLine(Graphics g,
    int x0, int y0, int y1, int dotSpacing) {
    for (int y = ((y0 + 1) / 2) * 2; y <= y1; y += dotSpacing) {
      g.drawLine(x0, y, x0, y);
    }
  }

  protected void drawFocusRect(Graphics g,
    int x, int y, int w, int h) {
    Color c = g.getColor();
    g.setColor(Color.black);

    drawDottedHLine(g, y, x, x + w - 1, 2);
    drawDottedVLine(g, x + w - 1, y, y + h - 1, 2);
    drawDottedHLine(g, y + h - 1, x, x + w - 1, 2);
    drawDottedVLine(g, x, y, y + h - 1, 2);
    g.setColor(c);
  }

  private void writeObject(ObjectOutputStream s)
    throws IOException {

    s.defaultWriteObject();
    s.writeObject(
      painter instanceof Serializable ? painter : null);
  }

  private void readObject(ObjectInputStream s)
    throws IOException, ClassNotFoundException {

    s.defaultReadObject();
    Object data = s.readObject();
    if (data instanceof ItemPainter) {
      painter = (ItemPainter)data;
    }
  }
}