package higa.jsqlplus;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

import java.text.DecimalFormat;

import java.util.Hashtable;
import java.util.Enumeration;

import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import higa.util.TextDialog;
import higa.util.JDBCUtil;
import higa.util.Resources;

public class Jsqlplus {

  public static final int NORMAL = 0;
  public static final int EXIT = 1;
  public static final int EDIT = 2;
  public static final int CONNECT = 3;
  public static final int PLSQL = 4;
  public static final int CALL = 5;
  public static final int DESCRIBE = 6;

  private static final Resources resources =
    new Resources("higa.jsqlplus.resources.login");

  private String driver =
    resources.getString("driver");

  private String connectionURL =
    resources.getString("connectionURL");

  private String userName =
    resources.getString("username");

  private String password =
    resources.getString("password");

  private transient Connection connection;

  //݂̃R}hi[obt@
  private StringBuffer buffer = new StringBuffer();
  //R}hE^Cv
  //(NORMAL,EXIT,EDIT,CONNECT,PLSQL,CALL,DESCRIBE)
  private int commandType;

  private Hashtable inParameter = new Hashtable();
  private Hashtable outParameter = new Hashtable();

  private BufferedReader in;

  private TextDialog resultDialog = new TextDialog();
  private TextDialog editDialog = new TextDialog();
  private ConnectDialog connectDialog = new ConnectDialog(
    driver, connectionURL, userName, password);

  public Jsqlplus() {
    resultDialog.setEditable(false);
    System.out.println();
    try {
      if (null == connection) {
       connect();
      }

      mainLoop();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public static void main(String[] args) {
    Jsqlplus jsqlplus = new Jsqlplus();
  }

  private void mainLoop() throws Exception {
    in = new BufferedReader(
      new InputStreamReader(System.in), 1);

    try {
      while (true) {
        try {
          getCommand();
          execute();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    } finally {
      disconnect();
      in.close();
    }
  }

  private void connect() throws Exception {
    connect(null);
  }

  private void connect(String s) throws Exception {
    if (s != null) {
      int index = s.indexOf('/');
      if (index < 0) {
        userName = s;
      } else {
        userName = s.substring(0, index).trim();
        password = s.substring(index + 1).trim();
        _connect();
        return;
      }
    }

    if (null == connectDialog) {
      connectDialog = new ConnectDialog(
        driver, connectionURL, userName, password);
    } else {
      connectDialog.setDriver(driver);
      connectDialog.setConnectionURL(connectionURL);
      connectDialog.setUserName(userName);
      connectDialog.setPassword(password);
    }

    connectDialog.setVisible(true);

    if (connectDialog.isOK()) {
      driver = connectDialog.getDriver();
      connectionURL = connectDialog.getConnectionURL();
      userName = connectDialog.getUserName();
      password = connectDialog.getPassword();
      _connect();
    }
  }

  private void _connect() throws Exception {
    disconnect();
    Driver d = null;
    try {
      d = DriverManager.getDriver(connectionURL);
    } catch (Exception ex) {
      Class.forName(driver).newInstance();
    }

    connection = DriverManager.getConnection(
      connectionURL, userName, password);
      
    if(connection != null) {
      connection.setAutoCommit(false);
    }
  }

  private void disconnect() throws SQLException {
    if (connection != null) {
      connection.commit();
      connection.close();
      connection = null;
    }
  }

  private void getCommand() throws Exception {
    //sԍ
    int lineNo = 1;

    while (true) {
      if (1 == lineNo) {
        commandType = NORMAL;
        System.out.print("SQL> ");
      } else {
        System.out.print(getLineNoString(lineNo));
      }

      String line = in.readLine();

      if (line.equals("/")) {
        System.out.print(buffer);
        break;
      }

      if (line.equals("")) {
        if (commandType != PLSQL) {
          lineNo = 1;
        }
        continue;
      }

      if (line.equals(".")) {
        lineNo = 1;
        continue;
      }

      if (1 == lineNo) {
        setCommandType(line);

        if (EXIT == commandType) {
          exitProcess();
        } else if (EDIT == commandType) {
          editProcess();
          lineNo = 1;
          continue;
        } else if (CONNECT == commandType) {
          connectProcess(line);
          lineNo = 1;
          continue;
        }

        buffer.setLength(0);

        if (DESCRIBE == commandType) {
          describeProcess(line);
          break;
        }
      }

      buffer.append(line);
      if (checkTermination()) {
        if (CALL == commandType) {
          callProcess();
        }
        System.out.print(buffer);
        return;
      } else {
        lineNo++;
        continue;
      }
    }
  }

  private String getLineNoString(int lineNo) {
    StringBuffer sb = new StringBuffer();
    DecimalFormat df = new DecimalFormat("###");
    String s = df.format((long)lineNo);
    int blanks = 3 - s.length();
    for (int i = 0; i < blanks; i++) {
      sb.append(" ");
    }
    sb.append(s + " ");
    sb.setLength(5);
    return sb.toString();
  }

  private void setCommandType(String line) {
    String s = line.toLowerCase().trim();
    int index = s.indexOf("\n");
    if (index > 0) {
      s = s.substring(0, index);
    }

    if (s.equals("exit") ||
      s.equals("exit;")) {
      commandType = EXIT;
    } else if (s.equals("ed") ||
      s.equals("ed;") ||
      s.equals("edit") ||
      s.equals("edit;")) {
      commandType = EDIT;
    } else if (s.equals("connect") ||
      s.startsWith("connect ") ||
      s.startsWith("connect;")) {
      commandType = CONNECT;
    } else if (s.equals("declare") ||
      s.startsWith("declare ") ||
      s.equals("begin") ||
      s.startsWith("begin ")) {
      commandType = PLSQL;
    } else if (s.startsWith("desc ") ||
      s.startsWith("describe ")) {
      commandType = DESCRIBE;
    } else if (s.equals("call") ||
      s.startsWith("call ") ||
      s.indexOf(" call ") >= 0) {
      commandType = CALL;
    }
  }

  private void exitProcess() throws SQLException {
    disconnect();
    System.exit(0);
  }

  private void editProcess() {
    if (0 == buffer.length()) {
      buffer.append("\n");
    }

    editDialog.setText(buffer.toString() + "/");
    editDialog.setVisible(true);

    String s = editDialog.getText();
    buffer.setLength(0);
    buffer.append(s);

    setCommandType(s);
    checkTermination();
    System.out.print(buffer);
  }

  private void connectProcess(String s) throws Exception {
    if (s.endsWith(";")) {
      s = s.substring(0, s.length() - 1);
    }
    if (s.length() > 8) {
      connect(s.substring(8));
    } else {
      connect();
    }
  }

  private void describeProcess(String s) {
    int tablePos = s.indexOf(" ") + 1;
    if (s.length() <= tablePos) {
      return;
    }

    buffer.append("SELECT * FROM ");
    buffer.append(s.substring(tablePos));
    checkTermination();
  }

  private void callProcess() {
    buffer.insert(0, "{");
    buffer.append("}\n");
  }

  private boolean checkTermination() {
    int len = buffer.length();
    if (len == 0) {
      buffer.append("\n");
    } else if ('/' == buffer.charAt(len - 1) ||
      commandType != PLSQL &&
      ';' == buffer.charAt(len - 1)) {
      if (CALL == commandType) {
        buffer.setLength(len - 1);
        return true;
      } else {
        buffer.setCharAt(len - 1, '\n');
      }

      if (len > 1 &&
        '\n' == buffer.charAt(len - 2)) {
        buffer.setLength(len - 1);
      }
      return true;
    } else if ('\n' != buffer.charAt(len - 1)) {
      buffer.append("\n");
    }
    return false;
  }

  private void execute()
    throws IOException, SQLException {
    
    setParameter();
    ResultSet rs = JDBCUtil.execute(
      connection, buffer.toString(),
      inParameter, outParameter);

    if (rs != null) {
      switch (commandType) {
        case DESCRIBE:
          ResultSetMetaData rsmd = rs.getMetaData();
          resultDialog.setText(
            JDBCUtil.describe(rsmd));
          break;
        default:
          resultDialog.setText(
            JDBCUtil.getResultSetContents(rs));
          break;
      }
      resultDialog.setVisible(true);
    } else if (!outParameter.isEmpty()) {
      Enumeration keys = outParameter.keys();
      Enumeration elements = outParameter.elements();
      StringBuffer sb = new StringBuffer();
      while (keys.hasMoreElements()) {
        Integer integer = (Integer)keys.nextElement();
        int index = integer.intValue();
        String value = (String)elements.nextElement();
        sb.insert(0, "Parameter" + index +
          ":" + value + "\n");
      }
      resultDialog.setText(sb.toString());
      resultDialog.setVisible(true);
    }
  }

  private void setParameter() throws IOException {
    inParameter.clear();
    outParameter.clear();
    int offset = 0;
    int questionIndex = 0;
    int parameterIndex = 0;
    String s = buffer.toString();
    
    while ((offset = s.indexOf('?', offset) + 1) > 0) {
      questionIndex++;
      System.out.print("Parameter" + questionIndex +
        " Type[1:IN,2:OUT(default),3:INOUT,4:NOT Parameter] ");
      int type = 2;
      try {
        type = Integer.parseInt(in.readLine());
      } catch (Exception ex) {}

      if (1 == type || 2 == type || 3 == type) {
        parameterIndex++;
      } else {
        continue;
      }

      Integer index = new Integer(parameterIndex);

      if (1 == type || 3 == type) {
        System.out.print("Value: ");
        String value = in.readLine();
        inParameter.put(index, value);
      }

      if (2 == type || 3 == type) {
        outParameter.put(index, "");
      }
    }
  }
}