package higa.sql.dataset;

import java.io.Serializable;

import java.util.Hashtable;

import java.text.Format;

import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.SQLException;

public class ColumnManager implements Serializable {

  private static final int DATE_DISPLAYSIZE =
    (new Date(System.currentTimeMillis())).toString().length();

  private static final int TIME_DISPLAYSIZE =
    (new Time(System.currentTimeMillis())).toString().length();

  private static final int TIMESTAMP_DISPLAYSIZE =
    (new Timestamp(System.currentTimeMillis())).toString().length();

  private Column[] columns = new Column[0];
  private Hashtable columnNames =
    new Hashtable();

  public ColumnManager() {
  }

  public ColumnManager(Column[] columns) {
    setColumns(columns);
  }

  public ColumnManager(ResultSetMetaData rsmd){
    try {
      int count = rsmd.getColumnCount();
      Column[] myColumns = new Column[count];

      for (int i = 0; i < count; i++) {
        String name = rsmd.getColumnName(
          i + 1).toUpperCase();

        String label = rsmd.getColumnLabel(i + 1);
        int type = rsmd.getColumnType(i + 1);
        boolean nullable =
          rsmd.isNullable(i + 1) == 0 ?
          false : true;

        int defaultSize = 0;
        int displaySize =
          rsmd.getColumnDisplaySize(i + 1);

        switch (type) {
          case Types.DATE:
            defaultSize = DATE_DISPLAYSIZE;
            break;
          case Types.TIME:
            defaultSize = TIME_DISPLAYSIZE;
            break;
          case Types.TIMESTAMP:
            defaultSize = TIMESTAMP_DISPLAYSIZE;
            break;
        }

        displaySize = Math.max(displaySize, defaultSize);

        myColumns[i] = new Column(
          name, label, type, nullable, displaySize);
      }

      setColumns(myColumns);
    } catch (SQLException ex) {
      ex.printStackTrace();
    }
  }

  public void setColumns(Column[] columns) {
    if (columns == null) {
      columns = new Column[0];
    }

    this.columns = columns;

    columnNames.clear();
    int count = getColumnCount();
    for (int i = 0; i < count; i++) {
      columnNames.put(
        getColumnName(i), new Integer(i));
    }
  }

  public Column[] getColumns() {
    int count = getColumnCount();
    Column[] newColumns = new Column[count];
    System.arraycopy(columns, 0,
      newColumns, 0, count);

    return newColumns;
  }

  public int getColumnCount() {
    return columns.length;
  }

  public String getColumnName(int index) {
    return columns[index].getName();
  }

  public int getColumnIndex(String columnName) {
    Integer index =
      (Integer)columnNames.get(
      columnName.toUpperCase());

    if (index == null) {
      return -1;
    }

    return index.intValue();
  }

  public void setColumn(String columnName, Column column) {
    int index = getColumnIndex(columnName);
    setColumn(index, column);
  }

  public void setColumn(int index, Column column) {
    columns[index] = column;
  }

  public Column getColumn(String columnName) {
    int index = getColumnIndex(columnName);
    return getColumn(index);
  }

  public Column getColumn(int index) {
    return new Column(columns[index]);
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    int count = getColumnCount();
    for (int i = 0; i < count; i++) {
      sb.append(columns[i] + "\n");
    }

    if (count > 0) {
      return sb.toString();
    } else {
      return null;
    }
  }
}