package higa.sql.dataset;

public class ReadWriteRow extends ReadRow
  implements Cloneable {

  public ReadWriteRow() {}

  public ReadWriteRow(Object[] items) {
    super(items);
  }

  public ReadWriteRow(ReadRow rr) {
    super(rr);
  }

  public Object clone() {
    return new ReadWriteRow(this);
  }

  public synchronized void set(int columnIndex, Object data) {
    if (!checkColumn(columnIndex)) {
      Object[] newItems = new Object[columnIndex + 1];
      System.arraycopy(items, 0,
        newItems, 0, items.length);

      items = newItems;
    }

    items[columnIndex] = data;
  }

  public synchronized void add() {
    add(null);
  }

  public synchronized void add(Object data) {
    Object[] newItems =
      new Object[items.length + 1];

    System.arraycopy(items, 0,
      newItems, 0, items.length);

    newItems[items.length] = data;
    items = newItems;
  }

  public synchronized void remove(int columnIndex) {
    checkColumn(columnIndex);

    Object[] newItems = new Object[items.length - 1];
    if (columnIndex == 0) {
      System.arraycopy(items, 1,
        newItems, 0, newItems.length);
    } else if (columnIndex == newItems.length) {
      System.arraycopy(items, 0,
        newItems, 0, newItems.length);
    } else {
      System.arraycopy(items, 0,
        newItems, 0, columnIndex);

      System.arraycopy(items, columnIndex + 1,
        newItems, columnIndex, newItems.length - columnIndex);
    }

    items = newItems;
  }

  public synchronized void removeAll() {
    items = new Object[0];
  }
}