package higa.util;

import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

import java.util.EventListener;

public class EventMulticaster implements Serializable {

  private transient EventListener[] listeners;

  public final boolean hasListeners() {
    return listeners != null;
  }

  public final void dispatch(Dispatch e) {
    EventListener[] ref = this.listeners;

    if (ref != null) {
      int count = ref.length;
      for (int i = 0; i < count; i++) {
        e.dispatch(ref[i]);
      }
    }
  }

  public final boolean vetoableDispatch(
    VetoableDispatch e) {
    EventListener[] ref = this.listeners;

    try {
      if (ref != null) {
        int count = ref.length;
        for (int i = 0; i < count; i++) {
          e.vetoableDispatch(ref[i]);
        }
      }
    } catch (VetoException ex) {
      return false;
    }

    return true;
  }

  public final void exceptionDispatch(
    ExceptionDispatch e) throws Exception {

    EventListener[] ref = this.listeners;

    if (ref != null) {
      int count = ref.length;
      for (int i = 0; i < count; i++) {
        e.exceptionDispatch(ref[i]);
      }
    }
  }

  public final int find(EventListener listener) {
    if (listeners != null ) {
      int count = listeners.length;
      for (int i = 0; i < count; i++) {
        if (listeners[i] == listener) {
          return i;
        }
      }
    }

    return -1;
  }

  public synchronized final void add(
    EventListener listener) {

    if (find(listener) < 0) {
      EventListener[] newListeners;
      int count = null == listeners ? 0 :
        listeners.length;

      newListeners =
        new EventListener[count + 1];

      if (count > 0) {
        System.arraycopy(listeners, 0,
          newListeners, 0, count);
      }

      newListeners[count] = listener;
      listeners = newListeners;
    }
  }

  public synchronized final void remove(
    EventListener listener) {

    int index = find(listener);
    if (index > -1) {
      int count = listeners.length;
      if (count == 1) {
        listeners = null;
      } else {
        EventListener[] newListeners =
          new EventListener[count - 1];

        System.arraycopy(listeners, 0,
          newListeners, 0, index);

        if (index < count - 1) {
          System.arraycopy(
            listeners, index + 1,
            newListeners, index,
            count - 1 - index);
        }

        listeners = newListeners;
      }
    }
  }

  protected EventListener[] getListeners() {
    return listeners;
  }

  private void writeObject(ObjectOutputStream s)
    throws IOException {

    s.defaultWriteObject();

    if (hasListeners()) {
      int count = listeners.length;
      for (int i = 0; i < count; i++) {
        if (listeners[i] instanceof Serializable) {
          try {
            s.writeObject(listeners[i]);
          } catch (Exception ex) {}
        }
      }
    }

    s.writeObject(null);
  }

  private void readObject(ObjectInputStream s)
    throws IOException, ClassNotFoundException {

    s.defaultReadObject();
    Object o = null;
    while ((o = s.readObject()) != null) {
      add((EventListener)o);
    }
  }
}
